@@ -35,7 +35,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 2008 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode2.c,v 1.4 2013/01/02 17:01:43 abe Exp $";
+static char *rcsid = "$Id: dnode2.c,v 1.5 2014/10/13 22:25:07 abe Exp $";
 #endif
 
 
@@ -135,4 +135,19 @@ readzfsnode(za, zi, vr)
 
 	return((char *)NULL);
 }
+
+
+
+
+# if	defined(__GNUC__) && defined(HAS_CV_TIMEDWAIT_SBT)
+/*
+ * A gcc work-around
+ */
+
+int     _cv_timedwait_sbt(struct cv *cvp, struct lock_object *lock,       
+            sbintime_t sbt, sbintime_t pr, int flags)
+{
+	return(0);
+}
+# endif	/* defined(__GNUC__) && HAS_CV_TIMEDWAIT_SBT */
 #endif	/* defined(HAS_ZFS) */
