@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.17 2013/01/02 17:01:43 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.18 2014/10/13 22:25:07 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -120,7 +120,14 @@ gather_proc_info()
 	struct filedesc fd;
 	int i, nf;
 	MALLOC_S nb;
-	static struct file **ofb = NULL;
+
+#if	defined(HAS_FILEDESCENT)
+	typedef struct filedescent ofb_t;
+#else	/* !defined(HAS_FILEDESCENT) */
+	typedef struct file* ofb_t;
+#endif	/* defined(HAS_FILEDESCENT) */
+
+	static ofb_t *ofb = NULL;
 	static int ofbb = 0;
 	int pgid, pid;
 	int ppid = 0;
@@ -137,10 +144,10 @@ gather_proc_info()
 	struct kinfo_proc *p;
 #endif	/* FREEBSDV<2000 */
 
-#if	defined(HASFSTRUCT)
+#if	defined(HASFSTRUCT) && !defined(HAS_FILEDESCENT)
 	static char *pof = (char *)NULL;
 	static int pofb = 0;
-#endif	/* defined(HASFSTRUCT) */
+#endif	/* defined(HASFSTRUCT) && !defiled(HAS_FILEDESCENT) */
 
 /*
  * Define socket and regular file conditional processing flags.
@@ -366,12 +373,12 @@ gather_proc_info()
 	 */
 	    if (!fd.fd_ofiles || (nf = fd.fd_nfiles) <= 0)
 		continue;
-	    nb = (MALLOC_S)(sizeof(struct file *) * nf);
+	    nb = (MALLOC_S)(sizeof(ofb_t) * nf);
 	    if (nb > ofbb) {
 		if (!ofb)
-		    ofb = (struct file **)malloc(nb);
+		    ofb = (ofb_t *)malloc(nb);
 		else
-		    ofb = (struct file **)realloc((MALLOC_P *)ofb, nb);
+		    ofb = (ofb_t *)realloc((MALLOC_P *)ofb, nb);
 		if (!ofb) {
 		    (void) fprintf(stderr, "%s: PID %d, no file * space\n",
 			Pn, p->P_PID);
@@ -382,7 +389,7 @@ gather_proc_info()
 	    if (kread((KA_T)fd.fd_ofiles, (char *)ofb, nb))
 		continue;
 
-#if	defined(HASFSTRUCT)
+#if	defined(HASFSTRUCT) && !defined(HAS_FILEDESCENT)
 	    if (Fsv & FSV_FG) {
 		nb = (MALLOC_S)(sizeof(char) * nf);
 		if (nb > pofb) {
@@ -400,20 +407,31 @@ gather_proc_info()
 		if (!fd.fd_ofileflags || kread((KA_T)fd.fd_ofileflags, pof, nb))
 		    zeromem(pof, nb);
 	    }
-#endif	/* defined(HASFSTRUCT) */
+#endif	/* defined(HASFSTRUCT) && !defined(HAS_FILEDESCENT) */
 
 	/*
 	 * Save information on file descriptors.
 	 */
 	    for (i = 0; i < nf; i++) {
-		if (ofb[i]) {
+
+#if	defined(HAS_FILEDESCENT)
+		if ((Cfp = ofb[i].fde_file))
+#else	/* !defined(HAS_FILEDESCENT) */
+		if ((Cfp = ofb[i]))
+#endif	/* defined(HAS_FILEDESCENT) */
+
+		{
 		    alloc_lfile(NULL, i);
-		    process_file((KA_T)(Cfp = ofb[i]));
+		    process_file((KA_T)Cfp);
 		    if (Lf->sf) {
 
 #if	defined(HASFSTRUCT)
 			if (Fsv & FSV_FG)
+# if	defined(HAS_FILEDESCENT)
+			    Lf->pof = (long)ofb[i].fde_flags;
+# else	/* !defined(HAS_FILEDESCENT) */
 			    Lf->pof = (long)pof[i];
+# endif	/* defined(HAS_FILEDESCENT) */
 #endif	/* defined(HASFSTRUCT) */
 
 			link_lfile();
