@@ -136,6 +136,16 @@ _PROTOTYPE(static struct pst_vm_status *read_vmreg,(struct pst_status *p,
 void
 gather_proc_info()
 {
+	short cckreg;			/* conditional status of regular file
+					 * checking:
+					 *     0 = unconditionally check
+					 *     1 = conditionally check */
+	short ckscko;			/* socket file only checking status:
+					 *     0 = none
+					 *     1 = check only socket files,
+					 *	   including TCP and UDP
+					 *	   streams with eXPORT data,
+					 *	   where supported */
 	int cwds, fd, *fds, fdsa, i, j, l, nf, np, rtds;
 	struct pst_fileinfo2 *f;
 	long flag;
@@ -170,6 +180,51 @@ gather_proc_info()
 	    fdsa = 0;
 	    fds = (int *)NULL;
 	}
+/*
+ * If only socket files have been selected, or socket files have been selected
+ * ANDed with other selection options, enable the skipping of regular files.
+ *
+ * If socket files and some process options have been selected, enable
+ * conditional skipping of regular file; i.e., regular files will be skipped
+ * unless they belong to a process selected by one of the specified options.
+ */
+	if (Selflags & SELNW) {
+
+	/*
+	 * Some network files selection options have been specified.
+	 */
+	    if (Fand || !(Selflags & ~SELNW)) {
+
+	    /*
+	     * Selection ANDing or only network file options have been
+	     * specified, so set unconditional skipping of regular files
+	     * and socket file only checking.
+	     */
+		cckreg = 0;
+		ckscko = 1;
+	    } else {
+
+	    /*
+	     * If ORed file selection options have been specified, or no ORed
+	     * process selection options have been specified, enable
+	     * unconditional file checking and clear socket file only checking.
+	     *
+	     * If only ORed process selection options have been specified,
+	     * enable conditional file skipping and socket file only checking.
+	     */
+		if ((Selflags & SELFILE) || !(Selflags & SELPROC))
+		    cckreg = ckscko = 0;
+		else
+		    cckreg = ckscko = 1;
+	    }
+	} else {
+
+	/*
+	 * No network file selection options were specified.  Enable
+	 * unconditional file checking and clear socket file only checking.
+	 */
+	    cckreg = ckscko = 0;
+	}
 /*
  * Examine proc structures and their associated information.
  */
@@ -185,13 +240,22 @@ gather_proc_info()
 	    p->pst_ucomm[PST_UCOMMLEN - 1] = '\0';
 	    if (is_cmd_excl(p->pst_ucomm, &pss, &sf))
 		continue;
+	    if (cckreg) {
+
+	    /*
+	     * If conditional checking of regular files is enabled, enable
+	     * socket file only checking, based on the process' selection
+	     * status.
+	     */
+		ckscko = (sf & SELPROC) ? 0 : 1;
+	    }
 	    alloc_lproc((int)p->pst_pid, (int)p->pst_pgrp, (int)p->pst_ppid,
 			(UID_ARG)p->pst_uid, p->pst_ucomm, (int)pss, (int)sf);
 	    Plf = (struct lfile *)NULL;
 	/*
 	 * Save current working directory information.
 	 */
-	    if (cwds
+	    if (!ckscko && cwds
 	    &&  IS_PSFILEID(&p->pst_cdir) && (p->pst_cdir.psf_fileid > 0)
 	    ) {
 		alloc_lfile(CWD, -1);
@@ -213,7 +277,7 @@ gather_proc_info()
 	/*
 	 * Save root directory information.
 	 */
-	    if (rtds
+	    if (!ckscko && rtds
 	    &&  IS_PSFILEID(&p->pst_rdir) && (p->pst_rdir.psf_fileid > 0)
 	    ) {
 		if (HvRtPsfid < 0)
@@ -244,7 +308,8 @@ gather_proc_info()
 	/*
 	 * Print information on the text files.
 	 */
-	    (void) process_text(p);
+	    if (!ckscko)
+		(void) process_text(p);
 	/*
 	 * Loop through user's files.
 	 */
@@ -310,7 +375,7 @@ gather_proc_info()
 #endif	/* defined(HASFSTRUCT) */
 
 	    /*
-	     * Save file offset.  _STAT64 should alwaus be defined, but just
+	     * Save file offset.  _PSTAT64 should alwaus be defined, but just
 	     * to be safe, check for it.
 	     */
 
@@ -325,6 +390,8 @@ gather_proc_info()
 	     */
 		switch (f->psf_ftype) {
 		case PS_TYPE_VNODE:
+		    if (ckscko || Selinet)
+			break;
 		    if ((na = read_det(&f->psf_fid, f->psf_hi_fileid,
 				       f->psf_lo_fileid, f->psf_hi_nodeid,
 				       f->psf_lo_nodeid, &pd)))
@@ -346,7 +413,7 @@ gather_proc_info()
 			if ((s = read_sock(f)))
 			    (void) process_socket(f, s);
 			else
-			    (void) process_stream(f);
+			    (void) process_stream(f, (int)ckscko);
 			break;
 		    default:
 			(void) snpf(Namech, Namechl,
@@ -355,7 +422,7 @@ gather_proc_info()
 		    }
 		    break;
 		case PS_TYPE_STREAMS:
-		    (void) process_stream(f);
+		    (void) process_stream(f, (int)ckscko);
 		    break;
 		case PS_TYPE_UNKNOWN:
 		    (void) snpf(Lf->type, sizeof(Lf->type), "UNKN");
