@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.23 2010/07/29 16:04:52 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.28 2014/10/13 22:25:58 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -80,11 +80,14 @@ static short Ckscko;			/* socket file only checking status:
  * Local function prototypes
  */
 
+_PROTOTYPE(static MALLOC_S alloc_cbf,(MALLOC_S len, char **cbf, MALLOC_S cbfa));
 _PROTOTYPE(static int get_fdinfo,(char *p, struct l_fdinfo *fi));
-_PROTOTYPE(static int getlinksrc,(char *ln, char *src, int srcl));
+_PROTOTYPE(static int getlinksrc,(char *ln, char *src, int srcl, char **rest));
+_PROTOTYPE(static int isefsys,(char *path, char *type, int l,
+			       efsys_list_t **rep, struct lfile **lfr));
 _PROTOTYPE(static int nm2id,(char *nm, int *id, int *idl));
-_PROTOTYPE(static int read_id_stat,(int ty, char *p, int id, char **cmd,
-				    int *ppid, int *pgid));
+_PROTOTYPE(static int read_id_stat,(char *p, int id, char **cmd, int *ppid,
+				    int *pgid));
 _PROTOTYPE(static void process_proc_map,(char *p, struct stat *s, int ss));
 _PROTOTYPE(static int process_id,(char *idp, int idpl, char *cmd, UID_ARG uid,
 				  int pid, int ppid, int pgid, int tid));
@@ -148,6 +151,29 @@ enter_cntx_arg(cntx)
 #endif	/* defined(HASSELINUX) */
 
 
+/*
+ * alloc_cbf() -- allocate a command buffer
+ */
+
+static MALLOC_S
+alloc_cbf(len, cbf, cbfa)
+	MALLOC_S len;				/* required length */
+	char **cbf;				/* current buffer */
+	MALLOC_S cbfa;				/* current buffer allocation */
+{
+	if (*cbf)
+	    *cbf = (char *)realloc((MALLOC_P *)*cbf, len);
+	else
+	    *cbf = (char *)malloc(len);
+	if (!*cbf) {
+	    (void) fprintf(stderr,
+		"%s: can't allocate command %d bytes\n", Pn, (int)len);
+	     Exit(1);
+	}
+	return(len);
+}
+
+
 /*
  * gather_proc_info() -- gather process information
  */
@@ -182,7 +208,7 @@ gather_proc_info()
 	    if (!(pidpath = (char *)malloc(pidpathl))) {
 		(void) fprintf(stderr,
 		    "%s: can't allocate %d bytes for \"%s/\"<pid>\n",
-		    Pn, pidpathl, PROCFS);
+		    Pn, (int)pidpathl, PROCFS);
 		Exit(1);
 	    }
 	    (void) snpf(pidpath, pidpathl, "%s/", PROCFS);
@@ -262,7 +288,7 @@ gather_proc_info()
 		{
 		    (void) fprintf(stderr,
 			"%s: can't allocate %d bytes for \"%s/%s/\"\n",
-			Pn, pidpathl, PROCFS, dp->d_name);
+			Pn, (int)pidpathl, PROCFS, dp->d_name);
 		    Exit(1);
 		}
 	    }
@@ -328,7 +354,7 @@ gather_proc_info()
 		    /*
 		     * Check the task state.
 		     */
-			rv = read_id_stat(1, tidpath, tid, &tcmd, &tppid,
+			rv = read_id_stat(tidpath, tid, &tcmd, &tppid,
 					  &tpgid);
 			if ((rv < 0) || (rv == 1))
 			    continue;
@@ -353,7 +379,7 @@ gather_proc_info()
 	 * with its tasks.
 	 */
 	    (void) make_proc_path(pidpath, n, &path, &pathl, "stat");
-	    if (((rv = read_id_stat(0, path, pid, &cmd, &ppid, &pgid)) >= 0) 
+	    if (((rv = read_id_stat(path, pid, &cmd, &ppid, &pgid)) >= 0) 
 	    &&   (rv != 1))
 	    {
 		tid = (Fand && ht && pidts && (Selflags & SELTASK)) ? pid : 0;
@@ -438,14 +464,19 @@ get_fdinfo(p, fi)
 
 
 static int
-getlinksrc(ln, src, srcl)
+getlinksrc(ln, src, srcl, rest)
 	char *ln;			/* link path */
 	char *src;			/* link source path return address */
 	int srcl;			/* length of src[] */
+	char **rest;			/* pointer to what follows the ':' in
+					 * the link source path (NULL if no
+					 * return requested) */
 {
 	char *cp;
 	int ll;
 
+	if (rest)
+	    *rest = (char *)NULL;
 	if ((ll = readlink(ln, src, srcl - 1)) < 1
 	||  ll >= srcl)
 	    return(-1);
@@ -455,6 +486,8 @@ getlinksrc(ln, src, srcl)
 	if ((cp = strchr(src, ':'))) {
 	    *cp = '\0';
 	    ll = strlen(src);
+	    if (rest)
+		*rest = cp + 1;
 	}
 	return(ll);
 }
@@ -568,7 +601,7 @@ make_proc_path(pp, pl, np, nl, sf)
 	    if (!cp) {
 		(void) fprintf(stderr,
 		    "%s: can't allocate %d bytes for %s%s\n",
-		    Pn, rl, pp, sf);
+		    Pn, (int)rl, pp, sf);
 		Exit(1);
 	    }
 	    *nl = rl;
@@ -580,6 +613,82 @@ make_proc_path(pp, pl, np, nl, sf)
 }
 
 
+/*
+ * isefsys() -- is path on a file system exempted with -e
+ *
+ * Note: alloc_lfile() must have been called in advance.
+ */
+
+static int
+isefsys(path, type, l, rep, lfr)
+	char *path;			/* path to file */
+	char *type;			/* unknown file type */
+	int l;				/* link request: 0 = report
+					 *               1 = link */
+	efsys_list_t **rep;		/* returned Efsysl pointer, if not
+					 * NULL */
+	struct lfile **lfr;		/* allocated struct lfile pointer */
+{
+	efsys_list_t *ep;
+	int ds, len;
+	struct mounts *mp;
+	char nmabuf[MAXPATHLEN + 1];
+
+	len = (int) strlen(path);
+	for (ep = Efsysl; ep; ep = ep->next) {
+
+	/*
+	 * Look for a matching exempt file system path at the beginning of
+	 * the file path.
+	 */
+	    if (ep->pathl > len)
+		continue;
+	    if (strncmp(ep->path, path, ep->pathl))
+		continue;
+	/*
+	 * If only reporting, return information as requested.
+	 */
+	    if (!l) {
+		if (rep)
+		    *rep = ep;
+		return(0);
+	    }
+	/*
+	 * Process an exempt file.
+	 */
+	    ds = 0;
+	    if ((mp = ep->mp)) {
+		if (mp->ds & SB_DEV) {
+		    Lf->dev = mp->dev;
+		    ds = Lf->dev_def = 1;
+		}
+		if (mp->ds & SB_RDEV) {
+		    Lf->rdev = mp->rdev;
+		    ds = Lf->rdev_def = 1;
+		}
+	    }
+	    if (!ds)
+		(void) enter_dev_ch("UNKNOWN");
+	    Lf->ntype = N_UNKN;
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%s",
+			(type ? type : "UNKN"));
+	    (void) enter_nm(path);
+	    (void) snpf(nmabuf, sizeof(nmabuf), "(%ce %s)",
+		ep->rdlnk ? '+' : '-', ep->path);
+	    nmabuf[sizeof(nmabuf) - 1] = '\0';
+	    (void) add_nma(nmabuf, strlen(nmabuf));
+	    if (Lf->sf) {
+		if (lfr)
+		    *lfr = Lf;
+		link_lfile();
+	    } else if (lfr)
+		*lfr = (struct lfile *)NULL;
+	    return(0);
+	}
+	return(1);
+}
+
+
 /*
  * nm2id() - convert a name to an integer ID
  */
@@ -701,7 +810,7 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid, tid)
 	int av;
 	static char *dpath = (char *)NULL;
 	static int dpathl = 0;
-	short enls, enss, lnk, oty, pn, pss, sf, tsf;
+	short efs, enls, enss, lnk, oty, pn, pss, sf;
 	int fd, i, ls, n, ss, sv;
 	struct l_fdinfo fi;
 	DIR *fdp;
@@ -709,12 +818,14 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid, tid)
 	static char *ipath = (char *)NULL;
 	static int ipathl = 0;
 	int j = 0;
+	struct lfile *lfr;
 	struct stat lsb, sb;
 	char nmabuf[MAXPATHLEN + 1], pbuf[MAXPATHLEN + 1];
 	static char *path = (char *)NULL;
 	static int pathl = 0;
 	static char *pathi = (char *)NULL;
 	static int pathil = 0;
+	char *rest;
 	int txts = 0;
 
 #if	defined(HASSELINUX)
@@ -726,8 +837,17 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid, tid)
  */
 	if (is_proc_excl(pid, pgid, uid, &pss, &sf, tid)
 	||  is_cmd_excl(cmd, &pss, &sf))
+	{
+
+#if	defined(HASEPTOPTS)
+	    if (!FpipeE)
+		return(1);
+#else	/* !defined(HASEPTOPTS) */
 	    return(1);
-	if (Cckreg) {
+#endif	/* defined(HASEPTOPTS) */
+
+	}
+	if (Cckreg && !FpipeE) {
 
 	/*
 	 * If conditional checking of regular files is enabled, enable
@@ -742,10 +862,11 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid, tid)
 /*
  * Process the ID's current working directory info.
  */
+	efs = 0;
 	if (!Ckscko) {
 	    (void) make_proc_path(idp, idpl, &path, &pathl, "cwd");
 	    alloc_lfile(CWD, -1);
-	    if (getlinksrc(path, pbuf, sizeof(pbuf)) < 1) {
+	    if (getlinksrc(path, pbuf, sizeof(pbuf), (char **)NULL) < 1) {
 		if (!Fwarn) {
 		    (void) memset((void *)&sb, 0, sizeof(sb));
 		    lnk = ss = 0;
@@ -758,25 +879,30 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid, tid)
 		    pn = 0;
 	    } else {
 		lnk = pn = 1;
-		ss = SB_ALL;
-		if (HasNFS) {
-		    if ((sv = statsafely(path, &sb)))
+		if (Efsysl && !isefsys(pbuf, "UNKNcwd", 1, NULL, &lfr)) {
+		    efs = 1;
+		    pn = 0;
+		} else {
+		    ss = SB_ALL;
+		    if (HasNFS) {
+			if ((sv = statsafely(path, &sb)))
 			sv = statEx(pbuf, &sb, &ss);
-		} else
-		    sv = stat(path, &sb);
-		if (sv) {
-		    ss = 0;
-		    if (!Fwarn) {
-			(void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
-			    strerror(errno));
-			nmabuf[sizeof(nmabuf) - 1] = '\0';
-			(void) add_nma(nmabuf, strlen(nmabuf));
+		    } else
+			sv = stat(path, &sb);
+		    if (sv) {
+			ss = 0;
+			if (!Fwarn) {
+			    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
+				strerror(errno));
+			    nmabuf[sizeof(nmabuf) - 1] = '\0';
+			    (void) add_nma(nmabuf, strlen(nmabuf));
+			}
 		    }
 		}
 	    }
 	    if (pn) {
 		(void) process_proc_node(lnk ? pbuf : path,
-					 &sb, ss,
+					 path, &sb, ss,
 					 (struct stat *)NULL, 0);
 		if (Lf->sf)
 		    link_lfile();
@@ -785,13 +911,13 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid, tid)
 /*
  * Process the ID's root directory info.
  */
+	lnk = ss = 0;
 	if (!Ckscko) {
 	    (void) make_proc_path(idp, idpl, &path, &pathl, "root");
 	    alloc_lfile(RTD, -1);
-	    if (getlinksrc(path, pbuf, sizeof(pbuf)) < 1) {
+	    if (getlinksrc(path, pbuf, sizeof(pbuf), (char **)NULL) < 1) {
 		if (!Fwarn) {
 		    (void) memset((void *)&sb, 0, sizeof(sb));
-		    lnk = ss = 0;
 		    (void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
 			strerror(errno));
 		    nmabuf[sizeof(nmabuf) - 1] = '\0';
@@ -801,25 +927,29 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid, tid)
 		    pn = 0;
 	    } else {
 		lnk = pn = 1;
-		ss = SB_ALL;
-		if (HasNFS) {
-		    if ((sv = statsafely(path, &sb)))
-			sv = statEx(pbuf, &sb, &ss);
-		} else
-		    sv = stat(path, &sb);
-		if (sv) {
-		    ss = 0;
-		    if (!Fwarn) {
-			(void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
-			    strerror(errno));
-			nmabuf[sizeof(nmabuf) - 1] = '\0';
-			(void) add_nma(nmabuf, strlen(nmabuf));
+		if (Efsysl && !isefsys(pbuf, "UNKNrtd", 1, NULL, NULL))
+		    pn = 0;
+		else {
+		    ss = SB_ALL;
+		    if (HasNFS) {
+			if ((sv = statsafely(path, &sb)))
+			    sv = statEx(pbuf, &sb, &ss);
+		    } else
+			sv = stat(path, &sb);
+		    if (sv) {
+			ss = 0;
+			if (!Fwarn) {
+			    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
+				strerror(errno));
+			    nmabuf[sizeof(nmabuf) - 1] = '\0';
+			    (void) add_nma(nmabuf, strlen(nmabuf));
+			}
 		    }
 		}
 	    }
 	    if (pn) {
 		(void) process_proc_node(lnk ? pbuf : path,
-					 &sb, ss,
+					 path, &sb, ss,
 					 (struct stat *)NULL, 0);
 		if (Lf->sf)
 		    link_lfile();
@@ -828,12 +958,12 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid, tid)
 /*
  * Process the ID's execution info.
  */
+	lnk = ss = txts = 0;
 	if (!Ckscko) {
 	    (void) make_proc_path(idp, idpl, &path, &pathl, "exe");
 	    alloc_lfile("txt", -1);
-	    if (getlinksrc(path, pbuf, sizeof(pbuf)) < 1) {
+	    if (getlinksrc(path, pbuf, sizeof(pbuf), (char **)NULL) < 1) {
 		(void) memset((void *)&sb, 0, sizeof(sb));
-		lnk = ss = 0;
 		if (!Fwarn) {
 		    if ((errno != ENOENT) || uid) {
 			(void) snpf(nmabuf, sizeof(nmabuf), "(readlink: %s)",
@@ -846,29 +976,33 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid, tid)
 		    pn = 0;
 	    } else {
 		lnk = pn = 1;
-		ss = SB_ALL;
-		if (HasNFS) {
-		    if ((sv = statsafely(path, &sb))) {
-			sv = statEx(pbuf, &sb,  &ss);
-			if (!sv && (ss & SB_DEV) && (ss & SB_INO))
-			    txts = 1;
-		    }
-		} else
-		    sv = stat(path, &sb);
-		if (sv) {
-		    ss = 0;
-		    if (!Fwarn) {
-			(void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
-			    strerror(errno));
-			nmabuf[sizeof(nmabuf) - 1] = '\0';
-			(void) add_nma(nmabuf, strlen(nmabuf));
-		    }
-		} else
-		    txts = 1;
+		if (Efsysl && !isefsys(pbuf, "UNKNtxt", 1, NULL, NULL))
+		    pn = 0;
+		else {
+		    ss = SB_ALL;
+		    if (HasNFS) {
+			if ((sv = statsafely(path, &sb))) {
+			    sv = statEx(pbuf, &sb,  &ss);
+			    if (!sv && (ss & SB_DEV) && (ss & SB_INO))
+				txts = 1;
+			}
+		    } else
+			sv = stat(path, &sb);
+		    if (sv) {
+			ss = 0;
+			if (!Fwarn) {
+			    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
+				strerror(errno));
+			    nmabuf[sizeof(nmabuf) - 1] = '\0';
+			    (void) add_nma(nmabuf, strlen(nmabuf));
+			}
+		    } else
+			txts = 1;
+		}
 	    }
 	    if (pn) {
 		(void) process_proc_node(lnk ? pbuf : path,
-					 &sb, ss,
+					 path, &sb, ss,
 					 (struct stat *)NULL, 0);
 		if (Lf->sf)
 		    link_lfile();
@@ -950,7 +1084,7 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid, tid)
 		continue;
 	    (void) make_proc_path(dpath, i, &path, &pathl, fp->d_name);
 	    (void) alloc_lfile((char *)NULL, fd);
-	    if (getlinksrc(path, pbuf, sizeof(pbuf)) < 1) {
+	    if (getlinksrc(path, pbuf, sizeof(pbuf), &rest) < 1) {
 		(void) memset((void *)&sb, 0, sizeof(sb));
 		lnk = ss = 0;
 		if (!Fwarn) {
@@ -963,69 +1097,92 @@ process_id(idp, idpl, cmd, uid, pid, ppid, pgid, tid)
 		    pn = 0;
 	    } else {
 		lnk = 1;
-		if (HasNFS) {
-		    if (lstatsafely(path, &lsb)) {
-			(void) statEx(pbuf, &lsb, &ls);
-		        enls = errno;
-		    } else {
-			enls = 0;
-			ls = SB_ALL;
-		    }
-		    if (statsafely(path, &sb)) {
-			(void) statEx(pbuf, &sb, &ss);
-			enss = errno;
-		    } else {
-			enss = 0;
-			ss = SB_ALL;
-		    }
+		if (Efsysl && !isefsys(pbuf, "UNKNfd", 1, NULL, &lfr)) {
+		    efs = 1;
+		    pn = 0;
 		} else {
-		    ls = lstat(path, &lsb) ? 0 : SB_ALL;
-		    enls = errno;
-		    ss = stat(path, &sb) ? 0 : SB_ALL;
-		    enss = errno;
-		}
-		if (!ls && !Fwarn) {
-		    (void) snpf(nmabuf, sizeof(nmabuf), "lstat: %s)",
-			strerror(enls));
-		    nmabuf[sizeof(nmabuf) - 1] = '\0';
-		    (void) add_nma(nmabuf, strlen(nmabuf));
-		}
-		if (!ss && !Fwarn) {
-		    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
-			strerror(enss));
-		    nmabuf[sizeof(nmabuf) - 1] = '\0';
-		    (void) add_nma(nmabuf, strlen(nmabuf));
-		}
-		if (Ckscko) {
-		    if ((ss & SB_MODE) && ((sb.st_mode & S_IFMT) == S_IFSOCK))
+		    if (HasNFS) {
+			if (lstatsafely(path, &lsb)) {
+			    (void) statEx(pbuf, &lsb, &ls);
+			    enls = errno;
+			} else {
+			    enls = 0;
+			    ls = SB_ALL;
+			}
+			if (statsafely(path, &sb)) {
+			    (void) statEx(pbuf, &sb, &ss);
+			    enss = errno;
+			} else {
+			    enss = 0;
+			    ss = SB_ALL;
+			}
+		    } else {
+			ls = lstat(path, &lsb) ? 0 : SB_ALL;
+			enls = errno;
+			ss = stat(path, &sb) ? 0 : SB_ALL;
+			enss = errno;
+		    }
+		    if (!ls && !Fwarn) {
+			(void) snpf(nmabuf, sizeof(nmabuf), "lstat: %s)",
+			    strerror(enls));
+			nmabuf[sizeof(nmabuf) - 1] = '\0';
+			(void) add_nma(nmabuf, strlen(nmabuf));
+		    }
+		    if (!ss && !Fwarn) {
+			(void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
+			    strerror(enss));
+			nmabuf[sizeof(nmabuf) - 1] = '\0';
+			(void) add_nma(nmabuf, strlen(nmabuf));
+		    }
+		    if (Ckscko) {
+			if ((ss & SB_MODE)
+			&&  ((sb.st_mode & S_IFMT) == S_IFSOCK))
+			{
+			    pn = 1;
+			} else
+			    pn = 0;
+		    } else
 			pn = 1;
-		    else
-			pn = 0;
-		} else
-		    pn = 1;
+		}
 	    }
-	    if (pn) {
+	    if (pn || (efs && lfr && oty)) {
 		if (oty) {
 		    (void) make_proc_path(ipath, j, &pathi, &pathil,
 					  fp->d_name);
 		    if ((av = get_fdinfo(pathi, &fi)) & FDINFO_POS) {
-			ls |= SB_SIZE;
-			lsb.st_size = fi.pos;
+			if (efs) {
+			    if (Foffset) {
+				lfr->off = (SZOFFTYPE)fi.pos;
+				lfr->off_def = 1;
+			    }
+			} else {
+			    ls |= SB_SIZE;
+			    lsb.st_size = fi.pos;
+			}
 		    } else
 			ls &= ~SB_SIZE;
 
 #if	!defined(HASNOFSFLAGS)
 		    if ((av & FDINFO_FLAGS) && (Fsv & FSV_FG)) {
-			Lf->ffg = (long)fi.flags;
-			Lf->fsv |= FSV_FG;
+			if (efs) {
+			    lfr->ffg = (long)fi.flags;
+			    lfr->fsv |= FSV_FG;
+			} else {
+			    Lf->ffg = (long)fi.flags;
+			    Lf->fsv |= FSV_FG;
+			}
 		     }
 # endif	/* !defined(HASNOFSFLAGS) */
 
 		}
-		process_proc_node(lnk ? pbuf : path,
-				  &sb, ss, &lsb, ls);
-		if (Lf->sf)
-		    link_lfile();
+		if (pn) {
+		    process_proc_node(lnk ? pbuf : path, path, &sb, ss, &lsb,
+				      ls);
+		    if ((Lf->ntype == N_ANON_INODE) && rest && *rest)
+			enter_nm(rest);
+		    if (Lf->sf)
+			link_lfile();
+		}
 	    }
 	}
 	(void) closedir(fdp);
@@ -1045,7 +1202,7 @@ process_proc_map(p, s, ss)
 {
 	char buf[MAXPATHLEN + 1], *ep, fmtbuf[32], **fp, nmabuf[MAXPATHLEN + 1];
 	dev_t dev;
-	int ds, en, i, mss, nf, sv;
+	int ds, efs, en, i, mss, nf, sv;
 	int eb = 6;
 	INODETYPE inode;
 	MALLOC_S len;
@@ -1058,6 +1215,7 @@ process_proc_map(p, s, ss)
 	    INODETYPE inode;
 	};
 	static struct saved_map *sm = (struct saved_map *)NULL;
+	efsys_list_t *rep;
 	static int sma = 0;
 	static char *vbuf = (char *)NULL;
 	static size_t vsz = (size_t)0;
@@ -1138,7 +1296,7 @@ process_proc_map(p, s, ss)
 		if (!sm) {
 		    (void) fprintf(stderr,
 			"%s: can't allocate %d bytes for saved maps, PID %d\n",
-			Pn, len, Lp->pid);
+			Pn, (int)len, Lp->pid);
 		    Exit(1);
 		}
 	    }
@@ -1146,17 +1304,25 @@ process_proc_map(p, s, ss)
 	    sm[ns++].inode = inode;
 	/*
 	 * Allocate space for the mapped file, then get stat(2) information
-	 * for it.
+	 * for it.  Skip the stat(2) operation if this is on an exempt file
+	 * system.
 	 */
 	    alloc_lfile("mem", -1);
-	    if (HasNFS) {
-		sv = statsafely(fp[6], &sb);
-	    } else
-		sv = stat(fp[6], &sb);
-	    if (sv) {
+	    if (Efsysl && !isefsys(fp[6], (char *)NULL, 0, &rep, NULL))
+		efs = sv = 1;
+	    else
+		efs = 0;
+	    if (!efs) {
+		if (HasNFS)
+		    sv = statsafely(fp[6], &sb);
+		else
+		    sv = stat(fp[6], &sb);
+	    }
+	    if (sv || efs) {
 		en = errno;
 	    /*
-	     * Applying stat(2) to the file failed, so manufacture a partial
+	     * Applying stat(2) to the file was not possible (file is on an
+	     * exempt file system) or stat(2) failed, so manufacture a partial
 	     * stat(2) reply from the process' maps file entry.
 	     *
 	     * If the file has been deleted, reset its type to "DEL"; otherwise
@@ -1169,7 +1335,7 @@ process_proc_map(p, s, ss)
 		mss = SB_DEV | SB_INO | SB_MODE;
 		if (ds)
 		    alloc_lfile("DEL", -1);
-		else {
+		else if (!efs && !Fwarn) {
 		    (void) snpf(nmabuf, sizeof(nmabuf), "(stat: %s)",
 			strerror(en));
 		    nmabuf[sizeof(nmabuf) - 1] = '\0';
@@ -1221,7 +1387,27 @@ process_proc_map(p, s, ss)
 	/*
 	 * Record the file's information.
 	 */
-	    process_proc_node(fp[6], &sb, mss, (struct stat *)NULL, 0);
+	    if (!efs)
+		process_proc_node(fp[6], fp[6], &sb, mss, (struct stat *)NULL,
+				  0);
+	    else {
+
+	    /*
+	     * If this file is on an exempt file system, complete the lfile
+	     * structure, but change its type and add the exemption note to
+	     * the NAME column.
+	     */
+		Lf->dev = sb.st_dev;
+		Lf->inode = (ino_t)sb.st_ino;
+		Lf->dev_def = Lf->inp_ty = 1;
+		(void) enter_nm(fp[6]);
+		(void) snpf(Lf->type, sizeof(Lf->type), "%s",
+			    (ds ? "UNKNdel" : "UNKNmem"));
+		(void) snpf(nmabuf, sizeof(nmabuf), "(%ce %s)",
+		    rep->rdlnk ? '+' : '-', rep->path);
+		nmabuf[sizeof(nmabuf) - 1] = '\0';
+		(void) add_nma(nmabuf, strlen(nmabuf));
+	    }
 	    if (Lf->sf)
 		link_lfile();
 	}
@@ -1239,8 +1425,7 @@ process_proc_map(p, s, ss)
  */
 
 static int
-read_id_stat(ty, p, id, cmd, ppid, pgid)
-	int ty;				/* type: 0 == PID, 1 == LWP */
+read_id_stat(p, id, cmd, ppid, pgid)
 	char *p;			/* path to status file */
 	int id;				/* ID: PID or LWP */
 	char **cmd;			/* malloc'd command name */
@@ -1249,11 +1434,11 @@ read_id_stat(ty, p, id, cmd, ppid, pgid)
 					 * type */
 {
 	char buf[MAXPATHLEN], *cp, *cp1, **fp;
+	int ch, cx, es, nf;
 	static char *cbf = (char *)NULL;
 	static MALLOC_S cbfa = 0;
 	FILE *fs;
-	MALLOC_S len;
-	int nf;
+	MALLOC_S nl;
 	static char *vbuf = (char *)NULL;
 	static size_t vsz = (size_t)0;
 /*
@@ -1262,69 +1447,89 @@ read_id_stat(ty, p, id, cmd, ppid, pgid)
  */
 	if (!(fs = open_proc_stream(p, "r", &vbuf, &vsz, 0)))
 	    return(-1);
-	cp = fgets(buf, sizeof(buf), fs);
-	(void) fclose(fs);
-	if (!cp)
-	    return(-1);
-/*
- * Separate the line into fields on white space separators.  Expect five fields
- * for a PID type and three for an LWP type.
- */
-	if ((nf = get_fields(buf, (char *)NULL, &fp, (int *)NULL, 0))
-	<  (ty ? 5 : 3))
-	{
+	if (!(cp = fgets(buf, sizeof(buf), fs))) {
+
+read_id_stat_exit:
+
+	    (void) fclose(fs);
 	    return(-1);
 	}
 /*
- * Convert the first field to an integer; its conversion must match the
- * ID argument.
+ * Skip to the first field, and make sure it is a matching ID.
  */
-	if (!fp[0] || (atoi(fp[0]) != id))
-	    return(-1);
-/*
- * Get the command name from the second field.  Strip a starting '(' and
- * an ending ')'.  Allocate space to hold the result and return the space
- * pointer.
- */
-	if (!(cp = fp[1]))
-	    return(-1);
-	if (cp && *cp == '(')
+	cp1 = cp;
+	while (*cp && (*cp != ' ') && (*cp != '\t'))
 	    cp++;
-	if ((cp1 = strrchr(cp, ')')))
-	    *cp1 = '\0';
-	if ((len = strlen(cp) + 1) > cbfa) {
-	     cbfa = len;
-	     if (cbf)
-		cbf = (char *)realloc((MALLOC_P *)cbf, cbfa);
-	     else
-		cbf = (char *)malloc(cbfa);
-	     if (!cbf) {
-		(void) fprintf(stderr,
-		    "%s: can't allocate %d bytes for command \"%s\"\n",
-		    Pn, cbfa, cp);
-		Exit(1);
-	     }
+	if (*cp)
+	    *cp = '\0';
+	if (atoi(cp1) != id)
+	    goto read_id_stat_exit;
+/*
+ * The second field should contain the command, enclosed in parentheses.
+ * If it also has embedded '\n' characters, replace them with '?' characters,
+ * accumulating command characters until a closing parentheses appears.
+ *
+ */
+	for (++cp; *cp && (*cp == ' '); cp++)
+		;
+	if (!cp || (*cp != '('))
+	    goto read_id_stat_exit;
+	cp++;
+/*
+ * Enter the command characters safely.  Supply them from the initial read
+ * of the stat file line, a '\n' if the initial read didn't yield a ')'
+ * command closure, or by reading the rest of the command a character at
+ * a time from the stat file.
+ */
+	for (cx = es = 0;;) {
+	    if (!es)
+		ch = *cp++;
+	    else {
+		if ((ch = fgetc(fs)) == EOF)
+		    goto read_id_stat_exit;
+	    }
+	    if (ch == ')')		/* ')' closes the command */
+		break;
+	    if ((cx + 2) > cbfa)
+		cbfa = alloc_cbf((cx + 2), &cbf, cbfa);
+	    cbf[cx] = ch;
+	    cx++;
+	    cbf[cx] = '\0';
+	    if (!es && !*cp)
+		es = 1;		/* Switch to fgetc() when a '\0' appears. */
 	}
-	(void) snpf(cbf, len, "%s", cp);
 	*cmd = cbf;
+/*
+ * Read the remainder of the stat line if it was necessary to read command
+ * characters individually from the stat file.
+ *
+ * Separate the reminder into fields.
+ */
+	if (es)
+	    cp = fgets(buf, sizeof(buf), fs);
+	(void) fclose(fs);
+	if (!cp || !*cp)
+	    return(-1);
+	if ((nf = get_fields(cp, (char *)NULL, &fp, (int *)NULL, 0)) < 3)
+	    return(-1);
 /*
  * Convert and return parent process (fourth field) and process group (fifth
  * field) IDs.
  */
-	if (fp[3] && *fp[3])
-	    *ppid = atoi(fp[3]);
+	if (fp[1] && *fp[1])
+	    *ppid = atoi(fp[1]);
 	else
 	    return(-1);
-	if (fp[4] && *fp[4])
-	    *pgid = atoi(fp[4]);
+	if (fp[2] && *fp[2])
+	    *pgid = atoi(fp[2]);
 	else
 	    return(-1);
 /*
  * Check the state in the third field.  If it is 'Z', return that indication.
  */
-	if (fp[2] && !strcmp(fp[2], "Z"))
+	if (fp[0] && !strcmp(fp[0], "Z"))
 	    return(1);
-	else if (fp[2] && !strcmp(fp[2], "T"))
+	else if (fp[0] && !strcmp(fp[0], "T"))
 	    return(2);
 	return(0);
 }
@@ -1376,7 +1581,7 @@ statEx(p, s, ss)
 	}
 	(void) strcpy(cb, p);
 /*
- * Trim trailing leaves from the end of the path one at a time and do s safe
+ * Trim trailing leaves from the end of the path one at a time and do a safe
  * stat() on each trimmed result.  Stop when a safe stat() succeeds or doesn't
  * fail because of EACCES or EPERM.
  */
