@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.34 2010/07/29 16:02:50 abe Exp $
+ * $Id: machine.h,v 1.37 2014/10/13 22:25:58 abe Exp $
  */
 
 
@@ -140,6 +140,13 @@
 /* #define	HASFIFONODE	1 */
 
 
+/*
+ * HASEOPT is defined for dialects that support the -e option
+ */
+
+#define	HASEOPT	1
+
+
 /*
  * HASFSINO is defined for those dialects that have the file system
  * inode element, fs_ino, in the lfile structure definition in lsof.h.
@@ -310,6 +317,13 @@
 /* #define	HASPIPENODE	1 */
 
 
+/*
+ * HASEPTOPTS is defined for dialects that support the +|-E options.
+ */
+
+#define		HASEPTOPTS	1
+
+
 /*
  * HASPMAPENABLED is defined when the reporting of portmapper registration
  * info is enabled by default.
@@ -581,7 +595,7 @@
 /* #define	USE_LIB_CKKV			1	   ckkv.c */
 /* #define	USE_LIB_COMPLETEVFS		1	   cvfs.c */
 /* #define	USE_LIB_FIND_CH_INO		1	   fino.c */
-#define	USE_LIB_IS_FILE_NAMED			1	/* isfn.c */
+/* #define	USE_LIB_IS_FILE_NAMED		1	   isfn.c */
 /* #define	USE_LIB_LKUPDEV			1	   lkud.c */
 /* #define	USE_LIB_PRINTDEVNAME		1	   pdvn.c */
 /* #define	USE_LIB_PROCESS_FILE		1	   prfp.c */
