@@ -34,7 +34,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: main.c,v 1.54 2010/07/29 15:59:28 abe Exp $";
+static char *rcsid = "$Id: main.c,v 1.56 2014/10/13 22:36:20 abe Exp $";
 #endif
 
 
@@ -112,14 +112,17 @@ main(argc, argv)
 	else
 	    Pn = argv[0];
 /*
- * Close all file descriptors above 2.
+ * Close enough file descriptors above 2 that library functions will have
+ * open descriptors.
  *
  * Make sure stderr, stdout, and stdin are open descriptors.  Open /dev/null
  * for ones that aren't.  Be terse.
  *
  * Make sure umask allows lsof to define its own file permissions.
  */
-	for (i = 3, n = GET_MAX_FD(); i < n; i++)
+	if ((MaxFd = (int) GET_MAX_FD()) < 53)
+	    MaxFd = 53;
+	for (i = 3; i < MaxFd; i++)
 	    (void) close(i);
 	while (((i = open("/dev/null", O_RDWR, 0)) >= 0) && (i < 2))
 	    ;
@@ -154,7 +157,7 @@ main(argc, argv)
  * Create option mask.
  */
 	(void) snpf(options, sizeof(options),
-	    "?a%sbc:D:d:%sf:F:g:hi:%s%slL:%sMnNo:Op:Pr:%ss:S:tT:u:UvVwx:%s%s%s",
+	    "?a%sbc:%sD:d:%s%sf:F:g:hi:%s%slL:%s%snNo:Op:Pr:%ss:S:tT:u:UvVwx:%s%s%s",
 
 #if	defined(HAS_AFS) && defined(HASAOPT)
 	    "A:",
@@ -168,6 +171,18 @@ main(argc, argv)
 	    "",
 #endif	/* defined(HASNCACHE) */
 
+#if	defined(HASEOPT)
+	    "e:",
+#else	/* !defined(HASEOPT) */
+	    "",
+#endif	/* defined(HASEOPT) */
+
+#if	defined(HASEPTOPTS)
+	    "E",
+#else	/* !defined(HASEPTOPTS) */
+	    "",
+#endif	/* defined(HASEPTOPTS) */
+
 #if	defined(HASKOPT)
 	    "k:",
 #else	/* !defined(HASKOPT) */
@@ -186,6 +201,12 @@ main(argc, argv)
 	    "",
 #endif	/* defined(HASMOPT) || defined(HASMNTSUP) */
 
+#if	defined(HASNORPC_H)
+	    "",
+#else	/* !defined(HASNORPC_H) */
+	    "M",
+#endif	/* defined(HASNORPC_H) */
+
 #if	defined(HASPPID)
 	    "R",
 #else	/* !defined(HASPPID) */
@@ -299,8 +320,8 @@ main(argc, argv)
 	    case 'C':
 		Fncache = (GOp == '-') ? 0 : 1;
 		break;
-#endif	/* defined(HASNCACHE) */
 
+#endif	/* defined(HASNCACHE) */
 	    case 'd':
 		if (GOp == '+') {
 		    if (enter_dir(GOv, 0))
@@ -334,6 +355,20 @@ main(argc, argv)
 
 		}
 		break;
+
+#if	defined(HASEOPT)
+	    case 'e':
+		if (enter_efsys(GOv, ((GOp == '+') ? 1 : 0)))
+		    err = 1;
+		break;
+#endif	/* defined(HASEOPT) */
+
+#if	defined(HASEPTOPTS)
+	    case 'E':
+		FpipeE = (GOp == '+') ? 2 : 1;
+		break;
+#endif	/* defined(HASEPTOPTS) */
+
 	    case 'f':
 		if (!GOv || *GOv == '-' || *GOv == '+') {
 		    Ffilesys = (GOp == '+') ? 2 : 1;
@@ -643,9 +678,12 @@ main(argc, argv)
 		break;
 #endif	/* defined(HASMOPT) || defined(HASMNTSUP) */
 
+#if	!defined(HASNORPC_H)
 	    case 'M':
 		FportMap = (GOp == '+') ? 1 : 0;
 		break;
+#endif	/* !defined(HASNORPC_H) */
+
 	    case 'n':
 		Fhost = (GOp == '-') ? 0 : 1;
 		break;
@@ -1037,6 +1075,35 @@ main(argc, argv)
 	    (void) fprintf(stderr, "%s: -x must accompany +d or +D\n", Pn);
 	    err++;
 	}
+
+#if	defined(HASEOPT)
+	if (Efsysl) {
+
+	/*
+	 * If there are file systems specified by -e options, check them.
+	 */
+	    efsys_list_t *ep;		/* Efsysl pointer */
+	    struct mounts *mp, *mpw;	/* local mount table pointers */
+
+	    if ((mp = readmnt())) {
+		for (ep = Efsysl; ep; ep = ep->next) {
+		    for (mpw = mp; mpw; mpw = mpw->next) {
+			if (!strcmp(mpw->dir, ep->path)) {
+			    ep->mp = mpw;
+			    break;
+			}
+		    }
+		    if (!ep->mp) {
+			(void) fprintf(stderr,
+			    "%s: \"-e %s\" is not a mounted file system.\n",
+			    Pn, ep->path);
+			err++;
+		    }
+		}
+	    }
+	}
+#endif	/* defined(HASEOPT) */
+
 	if (DChelp || err || Fhelp || fh || version)
 	    usage(err ? 1 : 0, fh, version);
 /*
@@ -1224,6 +1291,39 @@ main(argc, argv)
 		NcacheReload = 1;
 #endif	/* defined(HASNCACHE) */
 
+#if	defined(HASEPTOPTS)
+	    /*
+	     * If endpoint info has been requested, make sure it is coded for
+	     * printing.
+	     *
+	     * Lf contents must be preserved, since they may point to a
+	     * malloc()'d area, and since Lf is used throughout the print
+	     */
+		if (FpipeE) {
+		    lf = Lf;
+
+		/*
+		 * Check the files that have been selected for printing by
+		 * by some selection criterion other than being a pipe.
+		 */
+		    for (i = 0; i < Nlproc; i++) {
+			Lp = (Nlproc > 1) ? slp[i] : &Lproc[i];
+			if (Lp->pss && (Lp->pipe & PS_PIPE))
+			    (void) process_pinfo(0);
+		    }
+		/*
+		 * In a second pass, process unselected endpoint files,
+		 * possibly selecting them for printing.
+		 */
+		    for (i = 0; i < Nlproc; i++) {
+			Lp = (Nlproc > 1) ? slp[i] : &Lproc[i];
+			if (Lp->pipe & PS_PIPE_END)
+			    (void) process_pinfo(1);
+		    }
+		    Lf = lf;
+		}
+#endif	/* defined(HASEPTOPTS) */
+
 	    /*
 	     * Print the selected processes and count them.
 	     *
@@ -1250,6 +1350,11 @@ main(argc, argv)
 	 * If conditional repeat mode is in effect, see if it's time to exit.
 	 */
 	    if (RptTm) {
+
+#if	defined(HASEPTOPTS)
+		(void) clear_pinfo();
+#endif	/* defined(HASEPTOPTS) */
+
 		if (rc) {
 		    if (!n)
 			break;
