@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: print.c,v 1.51 2010/07/29 15:59:28 abe Exp $";
+static char *rcsid = "$Id: print.c,v 1.55 2013/01/02 17:14:59 abe Exp $";
 #endif
 
 
@@ -68,6 +68,13 @@ struct porttab {
 };
 
 
+#if	defined(HASNORPC_H)
+static struct porttab **Pth[2] = { NULL, NULL };
+						/* port hash buckets:
+						 * Pth[0] for TCP service names
+						 * Pth[1] for UDP service names
+						 */
+#else	/* !defined(HASNORPC_H) */
 static struct porttab **Pth[4] = { NULL, NULL, NULL, NULL };
 						/* port hash buckets:
 						 * Pth[0] for TCP service names
@@ -75,15 +82,20 @@ static struct porttab **Pth[4] = { NULL, NULL, NULL, NULL };
 						 * Pth[2] for TCP portmap info
 						 * Pth[3] for UDP portmap info
 						 */
+#endif	/* defined(HASNORPC_H) */
+
 #define HASHPORT(p)	(((((int)(p)) * 31415) >> 3) & (PORTHASHBUCKETS - 1))
 
 
+#if	!defined(HASNORPC_H)
 _PROTOTYPE(static void fill_portmap,(void));
+_PROTOTYPE(static void update_portmap,(struct porttab *pt, char *pn));
+#endif	/* !defined(HASNORPC_H) */
+
 _PROTOTYPE(static void fill_porttab,(void));
 _PROTOTYPE(static char *lkup_port,(int p, int pr, int src));
 _PROTOTYPE(static char *lkup_svcnam,(int h, int p, int pr, int ss));
 _PROTOTYPE(static int printinaddr,(void));
-_PROTOTYPE(static void update_portmap,(struct porttab *pt, char *pn));
 
 
 /*
@@ -104,6 +116,7 @@ endnm(sz)
 }
 
 
+#if !defined(HASNORPC_H)
 /*
  * fill_portmap() -- fill the RPC portmap program name table via a conversation
  *		     with the portmapper
@@ -267,6 +280,7 @@ fill_portmap()
 	}
 	clnt_destroy(c);
 }
+#endif	/* !defined(HASNORPC_H) */
 
 
 /*
@@ -468,7 +482,13 @@ lkup_port(p, pr, src)
  * If the hash buckets haven't been allocated, do so.
  */
 	if (!Pth[0]) {
+
+#if	defined(HASNORPC_H)
+	    nh = 2;
+#else	/* !defined(HASNORPC_H) */
 	    nh = FportMap ? 4 : 2;
+#endif	/* defined(HASNORPC_H) */
+
 	    for (h = 0; h < nh; h++) {
 		if (!(Pth[h] = (struct porttab **)calloc(PORTHASHBUCKETS,
 				sizeof(struct porttab *))))
@@ -483,6 +503,8 @@ lkup_port(p, pr, src)
 		}
 	    }
 	}
+
+#if	!defined(HASNORPC_H)
 /*
  * If we're looking up program names for portmapped ports, make sure the
  * portmap table has been loaded.
@@ -491,11 +513,15 @@ lkup_port(p, pr, src)
 	    (void) fill_portmap();
 	    pm++;
 	}
+#endif	/* !defined(HASNORPC_H) */
+
 /*
  * Hash the port and see if its name has been cached.  Look for a local
  * port first in the portmap, if portmap searching is enabled.
  */
 	h = HASHPORT(p);
+
+#if	!defined(HASNORPC_H)
 	if (!src && FportMap) {
 	    for (pt = Pth[pr+2][h]; pt; pt = pt->next) {
 		if (pt->port != p)
@@ -511,6 +537,8 @@ lkup_port(p, pr, src)
 		return(pt->name);
 	    }
 	}
+#endif	/* !defined(HASNORPC_H) */
+
 	for (pt = Pth[pr][h]; pt; pt = pt->next) {
 	    if (pt->port == p)
 		return(pt->name);
@@ -652,7 +680,7 @@ print_file()
 #if	defined(HASTASKS)
 	    if (TaskPrtFl)
 		(void) printf(" %*s", TidColW, TIDTTL);
-#endif	/* defined(HASTASKS)
+#endif	/* defined(HASTASKS) */
 
 #if	defined(HASZONES)
 	    if (Fzone)
@@ -1154,7 +1182,14 @@ addr_too_long:
 	 * Process the port number.
 	 */
 	    if (Lf->li[i].p > 0) {
-		if (Fport || FportMap) {
+
+		if (Fport
+
+#if	!defined(HASNORPC_H)
+		||  FportMap
+#endif	/* defined(HASNORPC_H) */
+
+		) {
 
 		/*
 		 * If converting port numbers to service names, or looking
@@ -1167,9 +1202,10 @@ addr_too_long:
 		 * loopback address 127.0.0.1.  (Test 2 may not always work
 		 * -- e.g., on hosts with multiple interfaces.)
 		 */
+#if	!defined(HASNORPC_H)
 		    if ((src = i) && FportMap) {
 
-#if	defined(HASIPv6)
+# if	defined(HASIPv6)
 			if (Lf->li[0].af == AF_INET6) {
 			    if (IN6_IS_ADDR_LOOPBACK(&Lf->li[i].ia.a6)
 			    ||  IN6_ARE_ADDR_EQUAL(&Lf->li[0].ia.a6,
@@ -1177,7 +1213,7 @@ addr_too_long:
 			    )
 				src = 0;
 			} else
-#endif	/* defined(HASIPv6) */
+# endif	/* defined(HASIPv6) */
 
 			if (Lf->li[0].af == AF_INET) {
 			    if (Lf->li[i].ia.a4.s_addr == htonl(INADDR_LOOPBACK)
@@ -1186,6 +1222,8 @@ addr_too_long:
 				src = 0;
 			}
 		    }
+#endif	/* !defined(HASNORPC_H) */
+
 		    if (strcasecmp(Lf->iproto, "TCP") == 0)
 			port = lkup_port(Lf->li[i].p, 0, src);
 		    else if (strcasecmp(Lf->iproto, "UDP") == 0)
@@ -1235,7 +1273,16 @@ addr_too_long:
 void
 print_init()
 {
+
+/*
+ * Preset standard values.
+ */
 	PrPass = (Ffield || Fterse) ? 1 : 0;
+	LastPid = -1;
+	TaskPrtFl = 0;
+/*
+ * Size columns by their titles.
+ */
 	CmdColW = strlen(CMDTTL);
 	DevColW = strlen(DEVTTL);
 	FdColW = strlen(FDTTL);
@@ -1252,7 +1299,6 @@ print_init()
 	    SzOffColW = strlen(OFFTTL);
 	else
 	    SzOffColW = strlen(SZOFFTTL);
-	TaskPrtFl = 0;
 
 #if	defined(HASTASKS)
 	TidColW = strlen(TIDTTL);
@@ -2163,8 +2209,7 @@ print_nma:
 	)) {
 	    if (ps)
 		putchar(' ');
-	    (void) print_tcptpi(1);
-	    return;
+	    (void) print_tcptpi(0);
 	}
 	if (nl)
 	    putchar('\n');
@@ -2748,6 +2793,7 @@ printunkaf(fam, ty)
 }
 
 
+#if	!defined(HASNORPC_H)
 /*
  * update_portmap() - update a portmap entry with its port number or service
  *		      name
@@ -2780,3 +2826,4 @@ update_portmap(pt, pn)
 	pt->nl = nl;
 	pt->ss = 1;
 }
+#endif	/* !defined(HASNORPC_H) */
