@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: proc.c,v 1.47 2013/01/02 17:14:59 abe Exp $";
+static char *rcsid = "$Id: proc.c,v 1.48 2014/10/13 22:36:20 abe Exp $";
 #endif
 
 
@@ -169,6 +169,10 @@ alloc_lfile(nm, num)
 	Lf->mnt_stat = (unsigned char)0;
 #endif	/* defined(HASMNTSTAT) */
 
+#if	defined(HASEPTOPTS)
+	Lf->pipend = 0;
+#endif	/* defined(HASEPTOPTS) */
+
 #if	defined(HASSOOPT)
 	Lf->lts.kai = Lf->lts.ltm = 0;
 	Lf->lts.opt = Lf->lts.qlen = Lf->lts.qlim = Lf->lts.pqlen
@@ -303,6 +307,10 @@ alloc_lproc(pid, pgid, ppid, uid, cmd, pss, sf)
 	Lp = &Lproc[Nlproc++];
 	Lp->pid = pid;
 
+#if	defined(HASEPTOPTS)
+	Lp->pipe = 0;
+#endif	/* defined(HASEPTOPTS) */
+
 #if	defined(HASTASKS)
 	Lp->tid = 0;
 #endif	/* defined(HASTASKS) */
@@ -882,6 +890,23 @@ link_lfile()
 {
 	if (Lf->sf & SELEXCLF)
 	    return;
+
+#if	defined(HASEPTOPTS)
+/*
+ * If endpoint info has been requested, clear the SELPINFO flag from the local
+ * file structure, since it was set only to insure this file would be linked.
+ * While this might leave no file selection flags set, a later call to the
+ * process_pinfo() function might set some. Also set the PS_PIPE flag for
+ * the process.
+ *
+ * Also set the SELPINFO flag for the process.
+ */
+	if (FpipeE) {
+	    Lf->sf &= ~SELPINFO;
+	    Lp->pipe |= PS_PIPE;
+	}
+#endif	/* defined(HASEPTOPTS) */
+
 	Lp->pss |= PS_SEC;
 	if (Plf)
 	    Plf->next = Lf;
@@ -898,6 +923,107 @@ link_lfile()
 }
 
 
+#if	defined(HASEPTOPTS)
+/*
+ * process_pinfo() -- process pipe info, adding it to selected files and
+ *		      selecting pipe end files (if requested)
+ */
+
+void
+process_pinfo(f)
+	int f;				/* function:
+					 *     0 == process selected pipe
+					 *     1 == process end point
+					 */
+{
+	struct lproc *ep;		/* pipe endpoint process */
+	struct lfile *ef;		/* pipe endpoint file */
+	int i;				/* temporary index */
+	char nma[1024];			/* name addition buffer */
+	pinfo_t *pp;			/* previous pipe info */
+	
+	if (!FpipeE)
+	    return;
+	for (Lf = Lp->file; Lf; Lf = Lf->next) {
+	    if ((Lf->ntype != N_FIFO) || (Lf->inp_ty != 1))
+		continue;
+	    pp = (pinfo_t *)NULL;
+	    switch(f) {
+	    case 0:
+
+	    /*
+	     * Process already selected pipe file.
+	     */
+		if (is_file_sel(Lp, Lf)) {
+
+		/*
+		 * This file has been selected by some criterion other than its
+		 * being a pipe.  Look up the pipe's endpoints.
+		 */
+		    do {
+			if ((pp = find_pendinfo(Lf, pp))) {
+
+			/*
+			 * This pipe endpoint is linked to the selected pipe
+			 * file.  Add its PID and FD to the name column
+			 * addition.
+			 */
+			    ep = &Lproc[pp->lpx];
+			    ef = pp->lf;
+			    for (i = 0; i < (FDLEN - 1); i++) {
+				if (ef->fd[i] != ' ')
+				    break;
+			    }
+			    (void) snpf(nma, sizeof(nma) - 1, "%d,%.*s,%s%c",
+				ep->pid, CmdLim, ep->cmd,&ef->fd[i],
+				ef->access);
+			    (void) add_nma(nma, strlen(nma));
+			    if (FpipeE == 2) {
+
+			    /*
+			     * Endpoint files have been selected, so mark this
+			     * one for selection later.
+			     */
+				ef->pipend = 1;
+				ep->pipe |= PS_PIPE_END;
+			    }
+			    pp = pp->next;
+			}
+		    } while (pp);
+		}
+		break;
+	    case 1:
+		if (!is_file_sel(Lp, Lf) && Lf->pipend) {
+
+		/*
+		 * This is an unselected end point file.  Select it and add
+		 * its end point information to its name column addition.
+		 */
+		    Lf->sf = Selflags;
+		    Lp->pss |= PS_SEC;
+		    do {
+			if ((pp = find_pendinfo(Lf, pp))) {
+			    ep = &Lproc[pp->lpx];
+			    ef = pp->lf;
+			    for (i = 0; i < (FDLEN - 1); i++) {
+				if (ef->fd[i] != ' ')
+				    break;
+			    }
+			    (void) snpf(nma, sizeof(nma) - 1, "%d,%.*s,%s%c",
+				ep->pid, CmdLim, ep->cmd, &ef->fd[i],
+				ef->access);
+			    (void) add_nma(nma, strlen(nma));
+			    pp = pp->next;
+			}
+		    } while (pp);
+		}
+		break;
+	    }
+	}
+}
+#endif	/* defined(HASEPTOPTS) */
+
+
 #if	defined(HASFSTRUCT)
 /*
  * print_fflags() - print interpreted f_flag[s]
@@ -1056,7 +1182,7 @@ print_proc()
 		    (void) printf("%c%s%c", LSOF_FID_LOGIN, cp, Terminator);
 	    }
 	    if (Terminator == '\0')
-	    putchar('\n');
+		putchar('\n');
 	}
 /*
  * Print files.
@@ -1066,25 +1192,29 @@ print_proc()
 		continue;
 	    rv = 1;
 	/*
-	 * If no field output selected, print dialects-specific formatted
+	 * If no field output selected, print dialect-specific formatted
 	 * output.
 	 */
 	    if (!Ffield) {
 		print_file();
 		continue;
 	    }
+	    lc = st = 0;
+	    if (FieldSel[LSOF_FIX_FD].st) {
+
+	    /*
+	     * Skip leading spaces in the file descriptor.  Print the field
+	     * identifier even if there are no characters after leading
+	     * spaces.
+	     */
+		for (cp = Lf->fd; *cp == ' '; cp++)
+		    ;
+		(void) printf("%c%s%c", LSOF_FID_FD, cp, Terminator);
+		lc++;
+	    }
 	/*
 	 * Print selected fields.
 	 */
-	    lc = st = 0;
-	    if (FieldSel[LSOF_FIX_FD].st) {
-		for (cp = Lf->fd; *cp == ' '; cp++)
-		    ;
-		if (*cp) {
-		    (void) printf("%c%s%c", LSOF_FID_FD, cp, Terminator);
-		    lc++;
-		}
-	    }
 	    if (FieldSel[LSOF_FIX_ACCESS].st) {
 		(void) printf("%c%c%c",
 		    LSOF_FID_ACCESS, Lf->access, Terminator);
